/*
	Pre-requisites:
		XML object (library), IO object (library).
	
	Requires:
		broadcaster id "ajax" in current document, attributes: [uri], [realm], [domain], [status]
	
	Usage:
		var d = new AJAX.doc();
		var c = d.newCommand('commandString')		// or: d.newCommand('commandString', 'commandLabel')
		c.newParam( 'param1Name', 'textValue' );
		c.newParam( 'param2Name', numericValue );
		d.send( functionObjectForSuccess, functionObjectForFailure|boolean );
*/

var AJAX = { 'internal' : {}, 'prop' : {} };

AJAX.getStrings = function() {
		if ( AJAX.prop.strings ) return AJAX.prop.strings;
		AJAX.prop.strings = document.getElementById('ajaxStrings');
		return AJAX.prop.strings;
	};

AJAX.getPrefPrefix = function() {
		if ( AJAX.prop.prefPrefix ) return AJAX.prop.prefPrefix;
		AJAX.prop.prefPrefix = 'thisapp.auth.';
		return AJAX.prop.prefPrefix;
	};

AJAX.getBroadcaster = function() {
		if ( AJAX.prop.broadcaster ) return AJAX.prop.broadcaster;
		AJAX.prop.broadcaster = document.getElementById('ajax');
		return AJAX.prop.broadcaster;
	};

AJAX.usePermissions = function() { return !! AJAX.getPermissions() };

AJAX.getPermissions = function() {
		if ( AJAX.prop.permissions ) return AJAX.prop.permissions;
		if ( AJAX.getSession() == '' ) return false;
		var d = IO.getUserChrome();
		d = d.path + IO.prop.sep;
		var e;
		try {
			d = IO.readXML( 'permissions.xml', d );
		} catch(e) {
			return false;
		}
		AJAX.prop.permissions = new Object();
		var els = XML.getElementChilds( d.documentElement );
		for ( var i = 0; i < els.length; i++ ) AJAX.prop.permissions[ els[i].nodeName ] = els[i].getAttribute('value') == 'true';
		return AJAX.prop.permissions;
	};

AJAX.getLoginUri = function() {
		if ( AJAX.prop.loginUri ) return AJAX.prop.loginUri;
		AJAX.prop.loginUri = IO.getPref( AJAX.getPrefPrefix() + 'login' );
		return AJAX.prop.loginUri;
	};

AJAX.setLoginUri = function( u ) {
		AJAX.prop.loginUri = u;
		IO.setPref( AJAX.getPrefPrefix() + 'login', u );
	};

AJAX.setUri = function( u ) {
		AJAX.prop.uri = u;
		IO.setPref( AJAX.getPrefPrefix() + 'ajax', u );
	};

AJAX.getUri = function() {
		if ( AJAX.prop.uri ) return AJAX.prop.uri;
		AJAX.prop.uri = IO.getPref( AJAX.getPrefPrefix() + 'ajax' );
		return AJAX.prop.uri;
	};

AJAX.getDocumentsUri = function() {
		if ( AJAX.prop.docs ) return AJAX.prop.docs;
		AJAX.prop.docs = IO.getPref( AJAX.getPrefPrefix() + 'docs' );
		return AJAX.prop.docs;
	};

AJAX.setDocumentsUri = function( u ) {
		AJAX.prop.docs = u;
		IO.setPref( AJAX.getPrefPrefix() + 'docs', u );
	};

AJAX.getRealm = function() {
		if ( AJAX.prop.realm ) return AJAX.prop.realm;
		AJAX.prop.realm = IO.getPref( AJAX.getPrefPrefix() + 'realm');
		return AJAX.prop.realm;
	};

AJAX.getBuild = function() {
		if ( AJAX.prop.build ) return AJAX.prop.build;
		AJAX.prop.build = IO.getPref( AJAX.getPrefPrefix() + 'build');
		return AJAX.prop.build;
	};

AJAX.getVersion = function() {
		if ( AJAX.prop.version ) return AJAX.prop.version;
		AJAX.prop.version = IO.getPref( AJAX.getPrefPrefix() + 'version');
		return AJAX.prop.version;
	};

AJAX.getDomain = function() {
		if ( AJAX.prop.domain ) return AJAX.prop.domain;
		AJAX.prop.domain = IO.getPref( AJAX.getPrefPrefix() + 'domain');
		return AJAX.prop.domain;
	};

AJAX.getLongDomain = function() {
		if ( AJAX.prop.longDomain ) return AJAX.prop.longDomain;
		AJAX.prop.longDomain = IO.getPref( AJAX.getPrefPrefix() + 'longDomain');
		return AJAX.prop.longDomain;
	};

AJAX.getUser = function() {
		if ( AJAX.prop.user) return AJAX.prop.user;
		AJAX.prop.user = IO.getPref( AJAX.getPrefPrefix() + 'user' );
		return AJAX.prop.user;
	};

AJAX.setUser = function(u) {
		AJAX.prop.user = u;
		return IO.setPref( AJAX.getPrefPrefix() + 'user', u );
	};

AJAX.getSession = function() {
		if ( AJAX.prop.session ) return AJAX.prop.session;
		AJAX.prop.session = IO.getPref( AJAX.getPrefPrefix() + 'session');
		return AJAX.prop.session;
	};

AJAX.setSession = function( s ) {
		AJAX.prop.session = s;
		IO.setPref( AJAX.getPrefPrefix() + 'session', s );
	};

AJAX.clearSession = function() {
		AJAX.prop.session = '';
		IO.delPref( AJAX.getPrefPrefix() + 'session' );
	};

AJAX.clearLogin = function() {
		IO.setPref( AJAX.getPrefPrefix() + 'user', '' );
		AJAX.logout();
	};

AJAX.logout = function() {
		AJAX.clearSession('');
		AJAX.prop.permissions = false;
	};

AJAX.getStatus = function() { return AJAX.getBroadcaster() ? AJAX.getBroadcaster().getAttribute('status') : '' }

AJAX.setStatus = function(s, p) {
		var ss;
		if (s == '') {
			ss = '';
		} else if (p) {
			ss = AJAX.getStrings().getFormattedString(s,p);
		} else {
			ss = AJAX.getStrings().getString(s);
		}
		var b = AJAX.getBroadcaster();
		if ( b ) b.setAttribute('status', ss);
	};

AJAX.setBusy = function( f ) { var b = AJAX.getBroadcaster(); if (b) b.setAttribute('busy', !!(f) ) }

AJAX.can = function( meth ) {
		var p = AJAX.getPermissions();
		return !!( p ) && !!( p[meth] );
	};

AJAX.control = function( ctrl ) {
		if ( ! AJAX.usePermissions() ) return;
		var doc = ctrl.ownerDocument;
		var pg  = ctrl.hasAttribute('bmPage') ? ctrl.getAttribute('bmPage') : false;
		var cmd = ctrl.hasAttribute('command') ? doc.getElementById( ctrl.getAttribute('command') ) : ctrl;
		var meths = cmd.hasAttribute('bmMethods') ? cmd.getAttribute('bmMethods').split(',') : [];
		var dis,deep = ctrl.hasAttribute('bmDeep') ? ctrl.getAttribute('bmDeep') : false;
		if ( deep ) {
			var bc = ctrl.firstChild;
			dis = doc.getElementById( bc.getAttribute('element') ).getAttribute( bc.getAttribute('attribute') ).toString() != deep;
		} else {
			dis = false;
		}
		var wrongPage = (MAIN) && (pg) ? ( MAIN.prop.activePg != pg ) : false;
		dis = dis || wrongPage;
		ctrl.disabled = dis;
		if ( ! ctrl.nodeName.match(/^(v|h|group)?box$/) ) dis = wrongPage;
		for ( var i = 0; i < meths.length; i++ ) dis = dis || ( ! AJAX.can( meths[i]) );
		ctrl.collapsed = dis;
	};

AJAX.deck = function( deck ) {
		var doc = deck.ownerDocument;
		var bc = deck.firstChild;
		var vals = deck.getAttribute('bmDeck').split(',');
		var cv = doc.getElementById( bc.getAttribute('element') ).getAttribute( bc.getAttribute('attribute') ).toString();
		for ( var i = 0; i < vals.length; i++ ) if ( vals[i] == cv ) deck.selectedIndex = i;
	};

AJAX.doc = function( options ) {
		if ( typeof options == 'undefined' ) options = new Object();
		if ( AJAX.getSession() == '' ) {
			if (document.getElementById('cmd_login')) { 
				document.getElementById('cmd_login').doCommand();
			} else if ( top.document.getElementById('cmd_login') ) {
				top.document.getElementById('cmd_login').doCommand();
			} else if ( opener.document.getElementById('cmd_login') ) {
				opener.document.getElementById('cmd_login').doCommand();
			} else {
				return false;
			}
		}
		this.doc = IO.newXML('root');
		if ( ! options.skipAuthentication ) AJAX.internal.addAuth( this.doc );
		this.getDocument = function() { return this.doc };
		this.newCommand = function(c,l) { return new AJAX.internal.command( this.doc, c, l ) };
		this.broadcaster = function( bn ) {
			var b = document.getElementById( bn );
			if ( b ) {
				this.doc.documentElement.setAttribute( 'broadcaster', bn );
				b.setAttribute( 'disabled', true );
			}
		};
		this.handleStatus = true;
		this.send = function( fOK, fKO ) { AJAX.internal.send( this.doc, fOK, fKO, this.handleStatus ) };
		this.debug = function() { IO.alert( 'URL: ' + AJAX.getUri(), IO.internal.xml2string( this.doc ) ) };
		return this;
	};

AJAX.internal.send = function( doc, fOK, fKO, handleStatus ) {
		AJAX.setStatus('sendingCmd');
		AJAX.setBusy( true );
		var f_ok = function(doc){ AJAX.internal.sendOK( doc, fOK, fKO, handleStatus ) };
		var f_ko = function(){ AJAX.internal.sendKO(fKO) };
		XML.post( AJAX.getUri(), f_ok, doc, f_ko, false );
	};

AJAX.internal.sendOK = function( doc, fOK, fKO, handleStatus ) {
		if ( doc.documentElement.hasAttribute('broadcaster') ) {
			var bc = document.getElementById( doc.documentElement.getAttribute('broadcaster') );
			if ( bc ) bc.setAttribute( 'disabled', false );
		}
		AJAX.setStatus('');
		AJAX.setBusy( false );
		var root = doc.documentElement;
		var status, root = doc.documentElement;
		if ( root.getAttribute('status') == 'xml' ) {
			status = -1;
			root.removeAttribute('status');
		} else {
			status = root.hasAttribute('status') ? parseInt( root.getAttribute('status') ) : 2;
		}
		if ( status > 1 ) {
			var errCode = root.getAttribute('errorCode');
			var errDesc = root.getAttribute('description');
			var str = AJAX.getStrings();
			if ( status > 99 ) {
				IO.alert( str.getFormattedString('fatalErrTitle', [ errCode ] ), str.getFormattedString('fatalErrBody', [ errDesc ] ) );
				IO.quit();
			} else if ( handleStatus && (status > 0 ) ) {
				IO.alert( str.getFormattedString('nonfatalErrTitle', [ errCode ] ), str.getFormattedString('nonfatalErrBody', [ errDesc ] ) );
				fKO( doc );
			} else {
				fKO( doc );
			}
		} else {
			fOK( doc );
		}
	};

AJAX.internal.sendKO = function( fKO ) {
		AJAX.setStatus('sendingErr');
		AJAX.setBusy( false );
		fKO();
	};

AJAX.internal.param = function( doc, parsNode, name, value, attName ) {
		if (typeof value == 'undefined' ) {
			value = '';
		} else if ( attName ) {
			value = value.toString().replace(/[ \t]+/g,' ').replace(/^ /,'').replace(/ $/,'');
		}
		var pObj = { 'name' : name };
		if ( attName ) pObj[attName] = value;
		this.node = XML.createNode('arrayitem','', pObj ,doc);
		this.getNode = function() { return this.node };
		parsNode.appendChild( this.node );
		return this;
	};

AJAX.internal.xmlParam = function( doc, parsNode, name, xml ) {
		if ( typeof xml == 'undefined' ) {
			xml = IO.newXML();
			xml.documentElement.setAttribute('error', 'Missing XML parameter');
		}
		var root = xml.documentElement ? xml.documentElement : xml;
		this.node = XML.createNode('arrayitem','', { 'name' : name, 'type' : 'xml' } ,doc);
		this.node.appendChild( root.cloneNode( true ) );
		this.getNode = function() { return this.node };
		parsNode.appendChild( this.node );
		return this;
	};

AJAX.internal.command = function( doc, command, label ) {
		if (typeof label == 'undefined') label = '';
		this.doc = doc;
		this.node = XML.createNode( 'arrayitem', '', {'command':command, 'label' : label }, doc );
		this.params = XML.createNode( 'params', '', false, doc );
		this.node.appendChild( this.params );
		this.doc.documentElement.appendChild( this.node );
		this.newParam = function( n, v ) { return new AJAX.internal.param( this.doc, this.params, n, v, 'value' ) };
		this.eParam = function( n, v ) { return new AJAX.internal.param( this.doc, this.params, n, v, false ) };
		this.xmlParam = function( n, x ) { return new AJAX.internal.xmlParam( this.doc, this.params, n, x ) };
		return this;
	};

AJAX.internal.addAuth = function( doc ) {
		var attrs = {
			'user'		: AJAX.getUser(),
			'session'	: AJAX.getSession(),
			'build'		: AJAX.getBuild(),
			'version'	: AJAX.getVersion()
		}
		doc.documentElement.appendChild( XML.createNode('auth', '', attrs, doc ) );
	};
