var TREE = { 'internal' : new Object() };

TREE.hasVScroll = function ( theTree) {
		var bo = theTree.boxObject;
		var firstrow = bo.getFirstVisibleRow();
		var lastrow  = bo.getLastVisibleRow();
		var numrows  = theTree.view.rowCount;
		return ( ( firstrow != 0 ) || ( lastrow < numrows ) );
	};

TREE.rowCount = function ( theTree ) { return theTree.view.rowCount };

TREE.buildSimple = function( doc, target, params, xsl ) {
		var e;
		try {
			if (typeof params == 'undefined') params = false;
			if (typeof target == 'string') target = document.getElementById( target );
			if (typeof xsl == 'undefined' ) xsl = 'results2simpletree.xsl';
			IO.xformin( doc, xsl, params, target );
		} catch(e) {
			IO.alert('TREE.buildSimple',e);
			throw( e );
		}
	};

TREE.buildChildrenSimple = function( doc, target, params, xsl ) {
		var e;
		try {
			if (typeof params == 'undefined') params = false;
			if (typeof target == 'string') target = document.getElementById( target );
			if (typeof xsl == 'undefined' ) xsl = 'results2simpletreechildren.xsl';
			if ( target.nodeName != 'treechildren' ){ 
				var xp = '//'+( target.hasAttribute('id') ? '*[@id="'+target.getAttribute('id')+'"]' : 'xul:tree')+'//xul:treechildren';
				target = IO.xpath(xp, target.ownerDocument)[0];
			}
			IO.xformin( doc, xsl, params, target );
		} catch(e) {
			IO.alert('TREE.buildChildrenSimple',e);
			throw( e );
		}
	};

TREE.buildSimpleTreeChildrenList = function( doc, target, command ) {
		var e;
		try {
			tree = document.getElementById( target );
			if (!tree ) return;
			if (!tree.hasAttribute('id')) return;

			var params = new Object();
			var tc  = XML.getFirstChildByName( tree, 'treecols' );
			if (typeof command != 'undefined' ) params.command = command;
			if ( tc.hasAttribute('keyfield') ) params.colkey = tc.getAttribute('keyfield');
			if ( tc.hasAttribute('keyprefix') ) params.keypfx = tc.getAttribute('keyprefix');
			var tcs = XML.getChildsByName( tc, 'treecol' );
			if (tcs){
				var ta = new Array();
				for ( var i = 0; i < tcs.length; i++ ) ta.push( tcs[i].getAttribute('field') );
				params.columns = ta.join(',');
			}
			var tgt = IO.xpath('//xul:tree[@id="'+tree.getAttribute('id')+'"]'+'//xul:treechildren', tree.ownerDocument)[0];
			IO.xformin( doc, 'simpleTreeChildrenList.xsl', params, tgt );
			if ( ! tree.hasAttribute('noSort') ) TREE.sort( tree );
		} catch(e) {
			IO.alert('TREE.buildSimpleTreeChildrenList',e);
			throw( e );
		}
	};


TREE.buildChildrenList = function( doc, target, params, xsl ) {
		var e;
		try {
			if (typeof params == 'undefined') params = new Object();
			if (typeof target == 'string') target = document.getElementById( target );
			if (typeof xsl == 'undefined' ) xsl = 'results2treechildrenlist.xsl';
			if ( target.nodeName != 'treechildren' ) target = IO.xpath( "//"+( target.hasAttribute('id') ? "*[@id='"+target.getAttribute('id')+"']": 'tree')+"//treechildren", target.ownerDocument)[0];
			var theTree = target.parentNode;
			var tc  = XML.getFirstChildByName( theTree, 'treecols' );
			if ( (! params.colkey) && tc.hasAttribute('keyfield') ) params.colkey = tc.getAttribute('keyfield');
			if ( ! params.columns ) {
				var tcs = XML.getChildsByName( tc, 'treecol' );
				var ta = new Array();
				for ( var i = 0; i < tcs.length; i++ ) ta.push( tcs[i].getAttribute('field') );
				params.columns = ta.join(',');
			}
			IO.xformin( doc, xsl, params, target );
			if ( ! theTree.hasAttribute('noSort') ) TREE.sort( theTree );
		} catch(e) {
			IO.alert('TREE.buildChildrenList',e);
			throw( e );
		}
	};

TREE.getSelectedRow = function( theTree ) {
		var ci = TREE.getSelectedItem( theTree );
		if ( ci ) {
			return XML.getFirstChildByName( ci, 'treerow' );
		} else {
			return false;
		}
	};

TREE.getSelectedItem = function( theTree ) {
		var cr = theTree && theTree.view && theTree.view.selection ? theTree.view.selection.currentIndex : -1;
		if ( cr < 0 ) return false;
		return theTree.contentView.getItemAtIndex(cr);
	};

TREE.getItemAt = function( theTree, position ) { return theTree.contentView.getItemAtIndex( position ) };

TREE.getTreeCols = function( theTree ) { return XML.getChildsByName( XML.getFirstChildByName( theTree, 'treecols' ), 'treecol' ) };

TREE.getTreeFields = function( theTree ) {
		var colsIn = TREE.getTreeCols( theTree );
		var colsOut = new Array();
		for ( var i = 0; i < colsIn.length; i++ ) if ( colsIn[i].hasAttribute('field') ) colsOut.push( colsIn[i].getAttribute('field') );
		return colsOut;
	};

TREE.isSelected = function( theTree ) { return theTree && theTree.view && theTree.view.selection ? ( theTree.currentIndex >= 0 ) : false }

TREE.isLeaf = function( theTree ) {
		if ( TREE.isSelected( theTree ) ) {
			return ! theTree.view.isContainer( theTree.currentIndex );
		} else {
			return false;
		}
	};

TREE.isContainer = function( theTree ) {
		if ( TREE.isSelected( theTree ) ) {
			return theTree.view.isContainer( theTree.currentIndex );
		} else {
			return false;
		}
	};

TREE.autoOpen = function( theTree ) {
		if ( TREE.isSelected( theTree) ) {
			var i = theTree.currentIndex;
			if ( theTree.view.isContainer( i )  && ! theTree.view.isContainerOpen(i) ) theTree.view.toggleOpenState( i );
		}
	};

TREE.autoToggle = function( theTree ) {
		if ( TREE.isSelected( theTree) ) {
			var i = theTree.currentIndex;
			if ( theTree.view.isContainer( i ) ) theTree.view.toggleOpenState( i );
		}
	};

TREE.getValue = function( theTree ) {
		var e;
		try {
			var ci = TREE.getSelectedItem( theTree );
			if (! ci) return false;
			return (ci.value) ? ci.value : ( ci.hasAttribute('value')  ? ci.getAttribute('value') : false );
		} catch(e) {
			IO.alert( 'TREE.getValue',  e );
			throw( e );
		}
	};

TREE.selectItem = function( theTree, theItem ) {
		TREE.clearSelection( theTree );
		var idx = theTree.contentView.getIndexOfItem( theItem );
		theTree.boxObject.ensureRowIsVisible( idx );
		theTree.view.selection.select( idx );
	};

TREE.selectByValue = function( theTree, theValue ) {
		if (typeof theValue == 'undefined' ) theValue= '* No Selected Item *';
		var n = IO.xpath("//xul:tree[@id='"+TREE.getId( theTree )+"']//xul:treeitem[@value='"+theValue+"']", theTree.ownerDocument);
		if ( n.length == 0 ) { return }
		TREE.clearSelection( theTree );
		var ancestors = IO.xpath("//xul:tree[@id='"+TREE.getId( theTree )+"']//xul:treeitem[.//xul:treeitem[@value='"+theValue+"']]", theTree.ownerDocument);
		for (var i=0; i < ancestors.length; i++) ancestors[i].setAttribute('open',true);
		n = n[0];
		var idx = theTree.contentView.getIndexOfItem( n );
		theTree.boxObject.ensureRowIsVisible( idx );
		if (theTree.view.isContainer(idx) && !theTree.view.isContainerOpen(idx)) theTree.view.toggleOpenState(idx);
		theTree.view.selection.select( idx );
	};

TREE.collpase = function( theTree ) { TREE.setExpandStatus( theTree, false ) };
TREE.expand = function( theTree ) { TREE.setExpandStatus( theTree, true ) };

TREE.setExpandStatus = function( theTree, status ) {
		status = !!status;
		var nodes = IO.xpath("//xul:tree[@id='"+TREE.getId( theTree )+"']//xul:treeitem[xul:treeitem]", theTree.ownerDocument);
		for (var i=0; i < ancestors.length; i++) ancestors[i].setAttribute('open', status );
	};

TREE.getId = function( theTree ) {
		if (! theTree.hasAttribute('id') ) TREE.internal.setTemporaryId( theTree );
		return theTree.getAttribute('id');
	};

TREE.clearSelection = function( theTree ) { theTree.view.selection.clearSelection() }

TREE.selectAt = function( theTree, index ) { theTree.view.selection.select( parseInt(index) ) }

TREE.getItemCell = function( node ) {
		if (!node) return false;
		var tr = XML.getFirstChildByName( node, 'treerow' );
		if ( ! tr ) return false;
		return XML.getFirstChildByName( tr, 'treecell' );
	};

TREE.internal.setTemporaryId = function( theTree ) { for ( var i = 0; !(theTree.hasAttribute('id')); i++ ) if (! theTree.ownerDocument.getElementById('treeDynamicId_'+i) ) theThree.setAttribute('id','treeDynamicId_'+i) }

TREE.filter = function( theTree, theText ) {
		var e;
		try {
			var f = theText.toString().replace(/'/g,'&#39;').replace(/</g,'&lt;').replace(/\s+/g,' ').toLowerCase();
			var tis = IO.xpath("//*[@id='"+TREE.getId(theTree)+"']//xul:treeitem", theTree.ownerDocument );
			var qt = 0;
			if ( f == '' ) {
				for ( var i = 0; i < tis.length; i++ ) tis[i].setAttribute('hidden', false );
			} else {
				TREE.clearSelection( theTree );
				for ( var i = 0; i < tis.length; i++ ) {
					var row = XML.getFirstChildByName( tis[i], 'treerow');
					var tcs = XML.getChildsByName( row, 'treecell' );
					var alltxt = '';
					for ( var j = 0; j < tcs.length; j++ ) alltxt += ' ' + tcs[j].getAttribute('label');
					alltxt = alltxt.toLowerCase().replace(/\s+/g,' ');
					var isIn = alltxt.indexOf(f) >= 0;
					if (isIn) qt++;
					tis[i].setAttribute('hidden', ! isIn );
				}
			}
			if ( qt == 1 ) {
				TREE.selectAt( theTree, 0 );
				theTree.focus();
			}
		} catch(e) {
			IO.alert( 'TREE.filter',  e );
			throw( e );
		}
	};

TREE.sort = function( theNode ) {
		var e;
		try {
			var theTree, treeCol, treeCols, sortDir;
			if ( theNode.nodeName == 'treechildren') theNode = theNode.parentNode;
			if ( theNode.nodeName == 'treecol' ) {
				treeCol = theNode;
				theTree = treeCol.parentNode.parentNode;
				treeCols = XML.getChildsByName( treeCol.parentNode, 'treecol' );
				sortDir = ( treeCol.hasAttribute('sortDirection') && ( treeCol.getAttribute('sortDirection')=='descending') ) ? 'ascending' : 'descending';
			} else if ( theNode.nodeName == 'tree' ) {
				theTree = theNode;
				treeCols = XML.getChildsByName( XML.getFirstChildByName( theTree, 'treecols' ), 'treecol' );
				treeCol = false;
				for ( var i = 0; i < treeCols.length; i++ ) {
					var tc = treeCols[i];
					if ( tc.hasAttribute('sortActive') && (tc.getAttribute('sortActive')=='true') ) {
						treeCol = tc;
						sortDir = treeCol.hasAttribute('sortDirection') ? treeCol.getAttribute('sortDirection') : 'ascending';
					}
					if ( tc.hasAttribute('field') && (! tc.hasAttribute('onclick')) && (! theTree.hasAttribute('noUserSort') )) tc.setAttribute('onclick','TREE.sort(this)');
				}
				if (! treeCol ) treeCol = treeCols[0];
			}
			var sortCol = treeCol.getAttribute('field');
			var sortIndex = false;
			for ( var i = 0; i < treeCols.length; i++ ) {
				var col = treeCols[i];
				if ( col.hasAttribute('field') && ( col.getAttribute('field') == sortCol ) ) {
					sortIndex = i;
					col.setAttribute( 'sortActive', true );
					col.setAttribute( 'sortDirection', sortDir );
				} else {
					if ( col.hasAttribute('sortActive') ) col.setAttribute('sortActive',false);
					if ( col.hasAttribute('sortDirection') ) col.removeAttribute('sortDirection');
				}
			}
			var target  = XML.getFirstChildByName( theTree, 'treechildren' );
			var doc = IO.newXML();
			var treeItems = XML.getChildsByName( target, 'treeitem' );
			for ( var i = 0; i < treeItems.length; i++ ) doc.documentElement.appendChild( treeItems[i].cloneNode(true) );
			var params = { 'sortIndex' : sortIndex +1, 'sortDir': sortDir == 'ascending' ? 'descending' : 'ascending', 'sortField' : sortCol };
			IO.xformin( doc, 'treelistsort.xsl', params, target );
			theTree.focus();
		} catch(e) {
			IO.alert('TREE.sort', e );
			throw( e );
		}
	};

