<?xml version="1.0" encoding="utf-8"?>

<!DOCTYPE xsl:stylesheet [
	<!ENTITY % mainDTD SYSTEM "chrome://main/locale/main.dtd" >
	%mainDTD;
	<!ENTITY % requestsDTD SYSTEM "chrome://main/locale/bookmanDlg.dtd" >
	%requestsDTD;
]>

<xsl:stylesheet
	version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns="http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul"
	>

<xsl:output method="xml" version="1.0" encoding="utf-8" indent="yes" />
	
	<xsl:param name="title"   select="'Tree'" />
	<xsl:param name="selFunc" select="''" />
	<xsl:param name="noroot"  select="''" /><!-- [OPTIONAL] if 'Y' fake box root node won't be generated, the result will start from tree -->
	<xsl:param name="rootName" select="''" /><!-- [OPTIONAL] if specified an addition root node is created -->
	<xsl:param name="rootValue" select="''" />
	<xsl:param name="treeId"  select="''" /><!-- [OPTIONAL] id of tree object, defaults to blank (no id) -->
	<xsl:param name="command" select="''" /><!-- [OPTIONAL] name of @command in resultset, defaults to every resultset -->
	<xsl:param name="label"   select="''" /><!-- [OPTIONAL] name of @label in resultset, defaults to every resultset -->
	<xsl:param name="seltype" select="'single'" />
	<xsl:param name="flex"    select="1" />
	<xsl:param name="class"   select="''" />
	
	<xsl:template match="/">
		<xsl:choose>
			<xsl:when test="$noroot='Y'">
				<xsl:apply-templates select="root" />
			</xsl:when>
			<xsl:otherwise>
				<box>
					<xsl:apply-templates select="root" />
				</box>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="root">
		<xsl:choose>
			<xsl:when test="string-length($command)!=0">
				<xsl:choose>
					<xsl:when test="string-length($label)!=0">
						<xsl:apply-templates select="arrayitem[(@command = $command) and (@label = $label)]/results" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:apply-templates select="arrayitem[@command = $command]/results" />
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:when test="string-length($label)!=0">
				<xsl:apply-templates select="arrayitem[@label=$label]/results" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="arrayitem/results" />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="results">
		<tree seltype="{$seltype}" hidecolumnpicker="true">
			<xsl:if test="string-length($flex)!=0">
				<xsl:attribute name="flex"><xsl:value-of select="$flex" /></xsl:attribute>
			</xsl:if>
			<xsl:if test="string-length($treeId)!=0">
				<xsl:attribute name="id"><xsl:value-of select="$treeId" /></xsl:attribute>
			</xsl:if>
			<xsl:if test="string-length($class)!=0">
				<xsl:attribute name="class"><xsl:value-of select="$class" /></xsl:attribute>
			</xsl:if>
			<xsl:if test="string-length($selFunc)!=0">
				<xsl:attribute name="onselect"><xsl:value-of select="$selFunc" /></xsl:attribute>
			</xsl:if>
			<treecols>
				<treecol label="{$title}" primary="true" flex="1" />
			</treecols>
			<treechildren>
				<xsl:variable name="children" select="arrayitem[string-length(@parent)=0]" />
				<xsl:choose>
					<xsl:when test="string-length($rootName)!=0">
						<xsl:variable name="hasChildren" select="count($children) != 0" />
						<treeitem>
							<xsl:if test="$hasChildren">
								<xsl:attribute name="container">true</xsl:attribute>
								<xsl:attribute name="open">true</xsl:attribute>
							</xsl:if>
							<treerow>
								<treecell label="{$rootName}" value="{$rootValue}">
									<xsl:attribute name="class">
										<xsl:choose>
											<xsl:when test="$hasChildren">treeNode</xsl:when>
											<xsl:otherwise>treeLeaf</xsl:otherwise>
										</xsl:choose>
									</xsl:attribute>
								</treecell>
							</treerow>
							<xsl:if test="$hasChildren">
								<treechildren>
									<xsl:apply-templates select="$children" />
								</treechildren>
							</xsl:if>
						</treeitem>
					</xsl:when>
					<xsl:otherwise>
						<xsl:apply-templates select="$children" />
					</xsl:otherwise>
				</xsl:choose>
			</treechildren>
		</tree>
	</xsl:template>
	
	<xsl:template match="arrayitem">
		<xsl:variable name="this" select="." />
		<xsl:variable name="children" select="../arrayitem[@parent=$this/@value]" />
		<xsl:variable name="hasChildren" select="count($children) != 0" />
		<treeitem value="{@value}">
			<xsl:if test="$hasChildren">
				<xsl:attribute name="container">true</xsl:attribute>
				<xsl:attribute name="open">false</xsl:attribute>
			</xsl:if>
			<treerow>
				<treecell label="{@name}">
					<xsl:attribute name="class">
						<xsl:choose>
							<xsl:when test="$hasChildren">treeNode</xsl:when>
							<xsl:otherwise>treeLeaf</xsl:otherwise>
						</xsl:choose>
					</xsl:attribute>
				</treecell>
			</treerow>
			<xsl:if test="$hasChildren">
				<treechildren>
					<xsl:apply-templates select="$children" />
				</treechildren>
			</xsl:if>
		</treeitem>
	</xsl:template>
	
</xsl:stylesheet>
