<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE xsl:stylesheet [
]>

<xsl:stylesheet
	version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns="http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul"
	>
	
	<xsl:output method="xml" version="1.0" encoding="utf-8" indent="yes" />
	
	<xsl:param name="columns" select="''" /><!-- [MANDATORY] names delimited list: COLNAME,COLNAME,... -->
	<xsl:param name="command" select="''" /><!-- [OPTIONAL] name of @command in resultset, defaults to every resultset -->
	<xsl:param name="label"   select="''" /><!-- [OPTIONAL] name of @label in resultset, defaults to every resultset -->
	<xsl:param name="colkey"  select="''" /><!-- [OPTIONAL] name of the column whose value will become treeitem value, no value if empty/missing -->
	<xsl:param name="keypfx"  select="''" /><!-- [OPTIONAL] name of the column whose value will become treeitem value, no value if empty/missing -->
	
	<xsl:template match="/">
		<treechildren>
			<xsl:apply-templates select="root" />
		</treechildren>
	</xsl:template>
	
	<xsl:template match="root">
		<xsl:choose>
			<xsl:when test="string-length($command)!=0">
				<xsl:choose>
					<xsl:when test="string-length($label)!=0">
						<xsl:apply-templates select="arrayitem[(@command = $command) and (@label = $label)]/results" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:apply-templates select="arrayitem[@command = $command]/results" />
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:when test="string-length($label)!=0">
				<xsl:apply-templates select="arrayitem[@label=$label]/results" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="arrayitem/results" />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="results">
		<xsl:apply-templates select="arrayitem">
			<xsl:with-param name="cols" select="$columns" />
		</xsl:apply-templates>
	</xsl:template>
	
	<xsl:template match="arrayitem">
		<xsl:param name="cols" />
		<treeitem>
			<xsl:if test="@*[name()=$colkey]">
				<xsl:attribute name="value">
					<xsl:value-of select="$keypfx" /><xsl:value-of select="@*[name()=$colkey]" />
				</xsl:attribute>
			</xsl:if>
			<treerow>
				<xsl:apply-templates select="." mode="treecell">
					<xsl:with-param name="cols"   select="$cols"      />
				</xsl:apply-templates>
			</treerow>
		</treeitem>
	</xsl:template>
	
	<xsl:template match="arrayitem" mode="treecell">
		<xsl:param name="cols"   />
		<xsl:variable name="colName">
			<xsl:call-template name="getFirst">
				<xsl:with-param name="text" select="$cols" />
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="colType">
			<xsl:value-of select="substring($colName,1,1)" />
		</xsl:variable>
		<treecell label="{@*[name()=$colName]}" field="{$colName}">
			<xsl:if test="string-length(@lproperties)!=0">
				<xsl:attribute name="properties">
					<xsl:value-of select="@lproperties" />
				</xsl:attribute>
			</xsl:if>
		</treecell>
		<xsl:variable name="moreCols">
			<xsl:call-template name="getMore">
				<xsl:with-param name="text" select="$cols" />
			</xsl:call-template>
		</xsl:variable>
		<xsl:if test="string-length($moreCols)!=0">
			<xsl:apply-templates select="." mode="treecell">
				<xsl:with-param name="cols"   select="$moreCols" />
			</xsl:apply-templates>
		</xsl:if>
	</xsl:template>
	
	<xsl:template name="getFirst">
		<xsl:param name="text" />
		<xsl:choose>
			<xsl:when test="contains($text,',')">
				<xsl:value-of select="substring-before($text,',')" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$text" />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template name="getMore">
		<xsl:param name="text" />
		<xsl:if test="contains($text,',')">
			<xsl:value-of select="substring-after($text,',')" />
		</xsl:if>
	</xsl:template>
	
</xsl:stylesheet>
