/*
	Pre-requisites:
		XML object (library), IO object (library).
	
	Requires:
		broadcaster id "ajax" in current document, attributes: [uri], [realm], [domain], [status]
	
	Usage:
		var d = new AAJAX.doc();
		var c = d.newCommand('commandString')		// or: d.newCommand('commandString', 'commandLabel')
		c.newParam( 'param1Name', 'textValue' );
		c.newParam( 'param2Name', numericValue );
		d.send( functionObjectForSuccess, functionObjectForFailure|boolean );
*/

var AAJAX = { 'internal' : {}, 'prop' : {} };

AAJAX.getStrings = function() {
		if ( AAJAX.prop.strings ) return AAJAX.prop.strings;
		AAJAX.prop.strings = document.getElementById('ajaxStrings');
		return AAJAX.prop.strings;
	};

AAJAX.getPrefPrefix = function() {
		if ( AAJAX.prop.prefPrefix ) return AAJAX.prop.prefPrefix;
		AAJAX.prop.prefPrefix = 'thisapp.auth.';
		return AAJAX.prop.prefPrefix;
	};

AAJAX.getBroadcaster = function() {
		if ( AAJAX.prop.broadcaster ) return AAJAX.prop.broadcaster;
		AAJAX.prop.broadcaster = document.getElementById('ajax');
		return AAJAX.prop.broadcaster;
	};

AAJAX.setUri = function( u ) {
		AAJAX.prop.uri = u;
		IO.setPref( AAJAX.getPrefPrefix() + 'altajax', u );
	};

AAJAX.getUri = function() {
		if ( AAJAX.prop.alturi ) return AAJAX.prop.alturi;
		AAJAX.prop.alturi = IO.getPref( AAJAX.getPrefPrefix() + 'altajax' );
		return AAJAX.prop.alturi;
	};

AAJAX.getDocumentsUri = function() {
		if ( AAJAX.prop.docs ) return AAJAX.prop.docs;
		AAJAX.prop.docs = IO.getPref( AAJAX.getPrefPrefix() + 'docs' );
		return AAJAX.prop.docs;
	};

AAJAX.setDocumentsUri = function( u ) {
		AAJAX.prop.docs = u;
		IO.setPref( AAJAX.getPrefPrefix() + 'docs', u );
	};

AAJAX.getRealm = function() {
		if ( AAJAX.prop.realm ) return AAJAX.prop.realm;
		AAJAX.prop.realm = IO.getPref( AAJAX.getPrefPrefix() + 'realm');
		return AAJAX.prop.realm;
	};

AAJAX.getBuild = function() {
		if ( AAJAX.prop.build ) return AAJAX.prop.build;
		AAJAX.prop.build = IO.getPref( AAJAX.getPrefPrefix() + 'build');
		return AAJAX.prop.build;
	};

AAJAX.getVersion = function() {
		if ( AAJAX.prop.version ) return AAJAX.prop.version;
		AAJAX.prop.version = IO.getPref( AAJAX.getPrefPrefix() + 'version');
		return AAJAX.prop.version;
	};

AAJAX.getDomain = function() {
		if ( AAJAX.prop.domain ) return AAJAX.prop.domain;
		AAJAX.prop.domain = IO.getPref( AAJAX.getPrefPrefix() + 'domain');
		return AAJAX.prop.domain;
	};

AAJAX.getLongDomain = function() {
		if ( AAJAX.prop.longDomain ) return AAJAX.prop.longDomain;
		AAJAX.prop.longDomain = IO.getPref( AAJAX.getPrefPrefix() + 'longDomain');
		return AAJAX.prop.longDomain;
	};

AAJAX.getStatus = function() { return AAJAX.getBroadcaster() ? AAJAX.getBroadcaster().getAttribute('status') : '' }

AAJAX.setStatus = function(s, p) {
		var ss;
		if (s == '') {
			ss = '';
		} else if (p) {
			ss = AAJAX.getStrings().getFormattedString(s,p);
		} else {
			ss = AAJAX.getStrings().getString(s);
		}
		var b = AAJAX.getBroadcaster();
		if ( b ) b.setAttribute('status', ss);
	};

AAJAX.setBusy = function( f ) { var b = AAJAX.getBroadcaster(); if (b) b.setAttribute('busy', !!(f) ) }

AAJAX.doc = function() {
		this.doc = IO.newXML('root');
		AAJAX.internal.addAuth( this.doc );
		this.getDocument = function() { return this.doc };
		this.newCommand = function(c,l) { return new AAJAX.internal.command( this.doc, c, l ) };
		this.handleStatus = true;
		this.send = function( fOK, fKO ) { AAJAX.internal.send( this.doc, fOK, fKO, this.handleStatus ) };
		return this;
	};

AAJAX.internal.send = function( doc, fOK, fKO, handleStatus ) {
		AAJAX.setStatus('sendingCmd');
		AAJAX.setBusy( true );
		var f_ok = function(doc){ AAJAX.internal.sendOK( doc, fOK, fKO, handleStatus ) };
		var f_ko = function(){ AAJAX.internal.sendKO(fKO) };
		XML.post( AAJAX.getUri(), f_ok, doc, f_ko, false );
	};

AAJAX.internal.sendOK = function( doc, fOK, fKO, handleStatus ) {
		AAJAX.setStatus('');
		AAJAX.setBusy( false );
		var root = doc.documentElement;
		var status = root.hasAttribute('status') ? parseInt( root.getAttribute('status') ) : 2;
		if ( status > 1 ) {
			var errCode = root.getAttribute('errorCode');
			var errDesc = root.getAttribute('description');
			var str = AAJAX.getStrings();
			if ( status > 99 ) {
				IO.alert( str.getFormattedString('fatalErrTitle', [ errCode ] ), str.getFormattedString('fatalErrBody', [ errDesc ] ) );
				IO.quit();
			} else if ( handleStatus && (status > 0 ) ) {
				IO.alert( str.getFormattedString('nonfatalErrTitle', [ errCode ] ), str.getFormattedString('nonfatalErrBody', [ errDesc ] ) );
				fKO( doc );
			} else {
				fKO( doc );
			}
		} else {
			fOK( doc );
		}
	};

AAJAX.internal.sendKO = function( fKO ) {
		AAJAX.setStatus('sendingErr');
		AAJAX.setBusy( false );
		fKO();
	};

AAJAX.internal.param = function( doc, parsNode, name, value, attName ) {
		if (typeof value == 'undefined' ) {
			value = '';
		} else if ( attName ) {
			value = value.toString().replace(/[ \t]+/g,' ').replace(/^ /,'').replace(/ $/,'');
		}
		var pObj = { 'name' : name };
		if ( attName ) pObj[attName] = value;
		this.node = XML.createNode('arrayitem','', pObj ,doc);
		this.getNode = function() { return this.node };
		parsNode.appendChild( this.node );
		return this;
	};

AAJAX.internal.command = function( doc, command, label ) {
		if (typeof label == 'undefined') label = '';
		this.doc = doc;
		this.node = XML.createNode( 'arrayitem', '', {'command':command, 'label' : label }, doc );
		this.params = XML.createNode( 'params', '', false, doc );
		this.node.appendChild( this.params );
		this.doc.documentElement.appendChild( this.node );
		this.newParam = function( n, v ) { return new AAJAX.internal.param( this.doc, this.params, n, v, 'value' ) };
		this.eParam = function( n, v ) { return new AAJAX.internal.param( this.doc, this.params, n, v, false ) };
		return this;
	};

AAJAX.internal.addAuth = function( doc ) {
		var attrs = {
			'build'		: AAJAX.getBuild(),
			'version'	: AAJAX.getVersion()
		}
		doc.documentElement.appendChild( XML.createNode('auth', '', attrs, doc ) );
	};
